import { Component, ViewChild, Input, NgModule } from '@angular/core';
import { FormsModule } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class DrawState {
}
class PencilState extends DrawState {
    constructor() {
        super(...arguments);
        this.currentCommand = new PencilCommand();
    }
    /**
     * Draws the next point in the path
     * @private
     * @param {?} ctx Canvas to be drawn on
     * @param {?} point Next point to draw
     * @return {?}
     */
    drawNextPoint(ctx, point) {
        ctx.beginPath();
        ctx.moveTo(this.lastCoord.x, this.lastCoord.y);
        ctx.lineTo(point.x, point.y);
        ctx.closePath();
        ctx.stroke();
    }
    /**
     * @return {?}
     */
    getName() {
        return 'pencil';
    }
    /**
     * @param {?} imageAnnotator
     * @param {?} ev
     * @return {?}
     */
    contactStart(imageAnnotator, ev) {
        this.lastCoord = ev.point;
        this.currentCommand.setColor((/** @type {?} */ (imageAnnotator.drawingCtx.fillStyle)));
    }
    /**
     * @param {?} imageAnnotator
     * @param {?} ev
     * @return {?}
     */
    contactMove(imageAnnotator, ev) {
        this.currentCommand.addPoint(ev.point);
        this.drawNextPoint(imageAnnotator.drawingCtx, ev.point);
        this.lastCoord = ev.point;
    }
    /**
     * @param {?} imageAnnotator
     * @param {?} ev
     * @return {?}
     */
    contactEnd(imageAnnotator, ev) {
        this.currentCommand.addPoint(ev.point);
        this.drawNextPoint(imageAnnotator.drawingCtx, ev.point);
        imageAnnotator.addCommand(this.currentCommand);
        this.currentCommand = new PencilCommand();
    }
    /**
     * @param {?} imageAnnotator
     * @return {?}
     */
    cleanUp(imageAnnotator) { }
}
class TextState extends DrawState {
    /**
     * @return {?}
     */
    getName() {
        return 'text';
    }
    /**
     * @private
     * @param {?} textEntry
     * @param {?} projectionFactor
     * @return {?}
     */
    generatePositionTextPairs(textEntry, projectionFactor) {
        /** @type {?} */
        const fullText = textEntry.getText();
        /** @type {?} */
        const lines = fullText.split('\n');
        /** @type {?} */
        const lineHeight = textEntry.getLineHeight() * projectionFactor;
        /** @type {?} */
        const positionTextPairs = [];
        for (let i = 0; i < lines.length; i++) {
            if (lines[i].trim() === '') {
                continue;
            }
            /** @type {?} */
            const position = { x: this.currentPosition.x, y: this.currentPosition.y + (i * lineHeight) };
            positionTextPairs.push({
                text: lines[i],
                position: position
            });
        }
        return positionTextPairs;
    }
    /**
     * @private
     * @param {?} imageAnnotator
     * @return {?}
     */
    recordCommandAndReset(imageAnnotator) {
        /** @type {?} */
        const positionTextPairs = this.generatePositionTextPairs(imageAnnotator.textEntry, imageAnnotator.projectionFactor);
        this.currentCommand.addPositionTextPairs(positionTextPairs);
        this.currentCommand.setColor((/** @type {?} */ (imageAnnotator.drawingCtx.fillStyle)));
        this.currentCommand.setFont(imageAnnotator.drawingCtx.font);
        this.currentCommand.draw(imageAnnotator.drawingCtx);
        imageAnnotator.addCommand(this.currentCommand);
        imageAnnotator.textEntry.clear();
        this.currentCommand = null;
    }
    /**
     * @param {?} imageAnnotator
     * @param {?} ev
     * @return {?}
     */
    contactStart(imageAnnotator, ev) {
        if (this.currentCommand && !imageAnnotator.textEntry.isEmpty()) {
            this.recordCommandAndReset(imageAnnotator);
            imageAnnotator.textEntry.hide();
        }
        else {
            this.currentCommand = new TextCommand();
            this.currentPosition = ev.point;
            imageAnnotator.textEntry.setPosition(ev.point.x / imageAnnotator.projectionFactor, ev.point.y / imageAnnotator.projectionFactor);
            imageAnnotator.textEntry.show();
            imageAnnotator.textEntry.focus();
            imageAnnotator.textEntry.onBlur()
                .then((/**
             * @param {?} _
             * @return {?}
             */
            _ => {
                if (this.currentCommand && !imageAnnotator.textEntry.isEmpty()) {
                    this.recordCommandAndReset(imageAnnotator);
                }
            }));
        }
    }
    /**
     * @param {?} imageAnnotator
     * @param {?} ev
     * @return {?}
     */
    contactMove(imageAnnotator, ev) {
        this.currentPosition = ev.point;
        imageAnnotator.textEntry.setPosition(ev.point.x / imageAnnotator.projectionFactor, ev.point.y / imageAnnotator.projectionFactor);
    }
    /**
     * @param {?} imageAnnotator
     * @param {?} ev
     * @return {?}
     */
    contactEnd(imageAnnotator, ev) {
        if (!this.currentCommand) {
            return;
        }
        imageAnnotator.textEntry.focus();
    }
    /**
     * @param {?} imageAnnotator
     * @return {?}
     */
    cleanUp(imageAnnotator) {
        if (this.currentCommand && !imageAnnotator.textEntry.isEmpty()) {
            this.recordCommandAndReset(imageAnnotator);
        }
        imageAnnotator.textEntry.hide();
    }
}
class PencilCommand {
    constructor() {
        this.pathArray = [];
    }
    /**
     * @param {?} point
     * @return {?}
     */
    addPoint(point) {
        this.pathArray.push(point);
    }
    /**
     * @param {?} color
     * @return {?}
     */
    setColor(color) {
        this.color = color;
    }
    /**
     * @param {?} ctx
     * @return {?}
     */
    draw(ctx) {
        /** @type {?} */
        const currentStrokeStyle = ctx.strokeStyle;
        ctx.strokeStyle = this.color;
        ctx.beginPath();
        for (let i = 1; i < this.pathArray.length; i++) {
            /** @type {?} */
            const coord = this.pathArray[i];
            /** @type {?} */
            const lastCoord = this.pathArray[i - 1];
            ctx.moveTo(lastCoord.x, lastCoord.y);
            ctx.lineTo(coord.x, coord.y);
            ctx.closePath();
        }
        ctx.stroke();
        ctx.strokeStyle = currentStrokeStyle;
    }
}
class TextCommand {
    constructor() {
        this.positionTextPairs = [];
    }
    /**
     * @param {?} pairs
     * @return {?}
     */
    addPositionTextPairs(pairs) {
        this.positionTextPairs = this.positionTextPairs.concat(pairs);
    }
    /**
     * @param {?} color
     * @return {?}
     */
    setColor(color) {
        this.color = color;
    }
    /**
     * @param {?} font
     * @return {?}
     */
    setFont(font) {
        this.font = font;
    }
    /**
     * @param {?} ctx
     * @return {?}
     */
    draw(ctx) {
        /** @type {?} */
        const currentFillStyle = ctx.fillStyle;
        /** @type {?} */
        const currentFont = ctx.font;
        ctx.fillStyle = this.color;
        ctx.font = this.font;
        for (const pair of this.positionTextPairs) {
            ctx.fillText(pair.text, pair.position.x, pair.position.y);
        }
        ctx.fillStyle = currentFillStyle;
        ctx.font = currentFont;
    }
}
class ClearCommand {
    /**
     * @param {?} canvasWidth
     * @param {?} canvasHeight
     */
    constructor(canvasWidth, canvasHeight) {
        this.canvasWidth = canvasWidth;
        this.canvasHeight = canvasHeight;
    }
    /**
     * @param {?} ctx
     * @return {?}
     */
    draw(ctx) {
        ctx.clearRect(0, 0, this.canvasWidth, this.canvasHeight);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContactEvent {
    /**
     * @param {?} type
     * @param {?} point
     */
    constructor(type, point) {
        this.type = type;
        this.point = point;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class EventTranslator {
    /**
     * @param {?} event
     * @param {?} canvasRect
     * @param {?} projectionFactor
     * @return {?}
     */
    translate(event, canvasRect, projectionFactor) {
        /** @type {?} */
        let singleEvent;
        // @ts-ignore
        if (window.TouchEvent && event instanceof TouchEvent) {
            singleEvent = event.changedTouches.item(0);
        }
        else {
            singleEvent = event;
        }
        /** @type {?} */
        const point = this.getCanvasPointFromEvent(singleEvent, canvasRect.left, canvasRect.top, projectionFactor);
        return new ContactEvent(this.getTypeFromEvent(event), point);
    }
    /**
     * @private
     * @param {?} ev
     * @param {?} offsetLeft
     * @param {?} offsetTop
     * @param {?} projectionFactor
     * @return {?}
     */
    getCanvasPointFromEvent(ev, offsetLeft, offsetTop, projectionFactor) {
        /** @type {?} */
        const point = {
            x: (ev.clientX - offsetLeft) * projectionFactor,
            y: (ev.clientY - offsetTop) * projectionFactor
        };
        return point;
    }
    /**
     * @private
     * @param {?} ev
     * @return {?}
     */
    getTypeFromEvent(ev) {
        /** @type {?} */
        let type;
        switch (ev.type) {
            case 'touchstart':
                type = 'start';
                break;
            case 'mousedown':
                type = 'start';
                break;
            case 'touchmove':
                type = 'move';
                break;
            case 'mousemove':
                type = 'move';
                break;
            case 'touchend':
                type = 'end';
                break;
            case 'mouseup':
                type = 'end';
                break;
            default:
                console.error(`aia-image-annotator: unrecognized contact event: ${ev.type}`);
        }
        return type;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULTS = {
    fontSize: '15px',
    fontFamily: 'Georgia',
    color: '#1218CE'
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FloatingTextEntryComponent {
    constructor() {
        this.currentText = '';
    }
    /**
     * @return {?}
     */
    getText() {
        return this.currentText;
    }
    /**
     * @return {?}
     */
    hide() {
        this.textBoxRef.nativeElement.style.display = 'none';
    }
    /**
     * @return {?}
     */
    show() {
        this.textBoxRef.nativeElement.style.display = 'block';
    }
    /**
     * @param {?} fontStr
     * @return {?}
     */
    setFont(fontStr) {
        this.textBoxRef.nativeElement.style.font = fontStr;
    }
    /**
     * @param {?} color
     * @return {?}
     */
    setColor(color) {
        this.textBoxRef.nativeElement.style.color = color;
    }
    /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    setPosition(x, y) {
        this.textBoxRef.nativeElement.style.top = y + 'px';
        this.textBoxRef.nativeElement.style.left = x + 'px';
    }
    /**
     * @return {?}
     */
    isEmpty() {
        return this.currentText.trim() === '';
    }
    /**
     * @return {?}
     */
    focus() {
        setTimeout((/**
         * @param {?} _
         * @return {?}
         */
        _ => {
            this.inputRef.nativeElement.focus();
        }), 0);
    }
    /**
     * @return {?}
     */
    clear() {
        this.currentText = '';
    }
    /**
     * @return {?}
     */
    onBlur() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            this.inputRef.nativeElement.addEventListener('blur', resolve);
        }));
    }
    /**
     * @return {?}
     */
    getLineHeight() {
        return this.getLineHeightHelper(this.textBoxRef.nativeElement);
    }
    /**
     * @private
     * @param {?} el
     * @return {?}
     */
    getLineHeightHelper(el) {
        /** @type {?} */
        let temp = document.createElement(el.nodeName);
        temp.setAttribute('style', `margin:0px;padding:0px;font-family:${el.style.fontFamily};font-size:${el.style.fontSize}`);
        temp.innerHTML = 'test';
        temp = el.parentNode.appendChild(temp);
        /** @type {?} */
        const ret = temp.clientHeight;
        temp.parentNode.removeChild(temp);
        return ret;
    }
}
FloatingTextEntryComponent.decorators = [
    { type: Component, args: [{
                selector: 'aia-floating-text-entry',
                template: "<div #textBox class=\"floating-text-entry\">\n    <textarea #input [(ngModel)]=\"currentText\"></textarea>\n    <p>{{currentText}}&nbsp;</p>\n</div>\n",
                styles: [".floating-text-entry{display:none;position:absolute;padding:0;margin:0;background-color:rgba(0,0,0,0);border:1px solid #5aa5be;text-align:left;white-space:pre}.floating-text-entry p{padding:0;margin:0}.floating-text-entry textarea{position:absolute;bottom:0;right:0;height:0;width:0;border:0;padding:0;background:rgba(0,0,0,0);resize:none}"]
            }] }
];
FloatingTextEntryComponent.propDecorators = {
    textBoxRef: [{ type: ViewChild, args: ['textBox',] }],
    inputRef: [{ type: ViewChild, args: ['input',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AiaImageAnnotatorComponent {
    constructor() {
        this.displayWidth = null;
        this.imageWidth = 0;
        this.imageHeight = 0;
        this.displayHeight = 0;
        /**
         * Multiply this to project from the DOM space to the canvas space
         */
        this.projectionFactor = 0;
        this._state = new PencilState();
        this._drawCommands = [];
        this._redoCommands = [];
        this.eventTranslator = new EventTranslator();
        this.mouseCurrentlyDown = false;
    }
    /**
     * @return {?}
     */
    get canvasRect() {
        return this.drawingCanvasRef.nativeElement.getBoundingClientRect();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.loadImageAndInitializeCanvas(this.image);
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.image) {
            // Full reset
            this._drawCommands = [];
            this._redoCommands = [];
            this.loadImageAndInitializeCanvas(this.image);
        }
        else {
            // Just colors and fonts
            this.setColorAndFont(this.color, this.fontSize, this.fontFamily);
        }
    }
    /**
       * BEGIN PUBLIC INTERFACE
       */
    /**
     * Sets current tool
     * @param {?} toolName Name of the tool
     * @return {?}
     */
    setTool(toolName) {
        if (toolName === this._state.getName()) {
            return;
        }
        this._state.cleanUp(this);
        switch (toolName) {
            case 'pencil':
                this._state = new PencilState();
                break;
            case 'text':
                this._state = new TextState();
                break;
            default:
                console.error(`Angular Image Annotator: Unrecognized tool name ${toolName}`);
                break;
        }
    }
    /**
     * Undo last action
     * @return {?}
     */
    undo() {
        if (!this._drawCommands.length) {
            return;
        }
        this._redoCommands.push(this._drawCommands.pop());
        this.clearCanvas();
        this.drawCommandsOnCanvas(this._drawCommands, this.drawingCtx);
    }
    /**
     * Redo last undone action if available
     * @return {?}
     */
    redo() {
        if (!this._redoCommands.length) {
            return;
        }
        this._drawCommands.push(this._redoCommands.pop());
        this.clearCanvas();
        this.drawCommandsOnCanvas(this._drawCommands, this.drawingCtx);
    }
    /**
     * Clears canvas undo-ably
     * @return {?}
     */
    clear() {
        /** @type {?} */
        const clearCommand = new ClearCommand(this.drawingCanvasRef.nativeElement.width, this.drawingCanvasRef.nativeElement.height);
        this.addCommand(clearCommand);
        this.drawCommandsOnCanvas([clearCommand], this.drawingCtx);
    }
    /**
     * Returns annotated image as data URI
     * @param {?=} type
     * @return {?}
     */
    getAnnotatedImage(type = 'image/png') {
        /** @type {?} */
        const mergeCanvas = this.mergeCanvasRef.nativeElement;
        mergeCanvas.style.width = this.drawingCanvasRef.nativeElement.width;
        mergeCanvas.style.height = this.drawingCanvasRef.nativeElement.height;
        /** @type {?} */
        const mergeCtx = mergeCanvas.getContext('2d');
        mergeCtx.drawImage(this.imageCanvasRef.nativeElement, 0, 0);
        mergeCtx.drawImage(this.drawingCanvasRef.nativeElement, 0, 0);
        return mergeCanvas.toDataURL(type);
    }
    /**
       * END PUBLIC INTERFACE
       */
    /**
     * Loads an image from an src and initializes the canvases. CLEARS ALL EXISTING ANNOTATIONS
     * @private
     * @param {?} image The src for the image to be annotated
     * @return {?}
     */
    loadImageAndInitializeCanvas(image) {
        if (!image) {
            console.log('aia-image-annotator: No image bound... skipping render');
            return;
        }
        /** @type {?} */
        const tempImage = new Image();
        tempImage.onload = (/**
         * @param {?} _
         * @return {?}
         */
        _ => {
            this.imageWidth = tempImage.width;
            this.imageHeight = tempImage.height;
            this.initializeDisplayDimensions();
            setTimeout((/**
             * @param {?} _
             * @return {?}
             */
            _ => {
                this.initializeCanvas(tempImage);
            }), 0);
        });
        tempImage.src = image;
    }
    /**
     * Sets projection factor, display width, and display height.
     * If display width hasn't been provided, it defaults to the image dimensions
     * @private
     * @return {?}
     */
    initializeDisplayDimensions() {
        if (this.displayWidth) {
            this.displayHeight = this.displayWidth * this.imageHeight / this.imageWidth;
            this.projectionFactor = this.imageHeight / this.displayHeight;
        }
        else {
            this.displayHeight = this.imageHeight;
            this.displayWidth = this.imageWidth;
            this.projectionFactor = 1;
        }
    }
    /**
     * Initializes the canvases (both drawing and image)
     * Sets canvas dimensions, loads contexts, etc.
     * @private
     * @param {?} img The image to load on the image canvas
     * @return {?}
     */
    initializeCanvas(img) {
        this.imageCtx = this.imageCanvasRef.nativeElement.getContext('2d');
        this.imageCtx.drawImage(img, 0, 0);
        this.drawingCtx = this.drawingCanvasRef.nativeElement.getContext('2d');
        this.drawingCtx.lineJoin = 'round';
        this.drawingCtx.lineCap = 'round';
        this.drawingCtx.lineWidth = 2;
        this.drawingCtx.textBaseline = 'hanging';
        this.setColorAndFont(this.color, this.fontSize, this.fontFamily);
    }
    /**
     * Sets the color and the font
     * @private
     * @param {?} color
     * @param {?} fontSize
     * @param {?} fontFamily
     * @return {?}
     */
    setColorAndFont(color, fontSize, fontFamily) {
        if (!this.drawingCtx) {
            return;
        }
        this.drawingCtx.strokeStyle = color || DEFAULTS.color;
        this.drawingCtx.fillStyle = color || DEFAULTS.color;
        this.textEntry.setColor(color || DEFAULTS.color);
        /** @type {?} */
        const fontString = `${fontSize || DEFAULTS.fontSize} ${fontFamily || DEFAULTS.fontFamily}`;
        this.drawingCtx.font = fontString;
        /** @type {?} */
        const fontParts = fontSize.match(/(.*)(px|pt)/);
        /** @type {?} */
        const adjustedFontSize = Math.floor(parseInt(fontParts[1], 10) / this.projectionFactor) + fontParts[2];
        this.textEntry.setFont(`${adjustedFontSize || DEFAULTS.fontSize} ${fontFamily || DEFAULTS.fontFamily}`);
    }
    /**
     * Clears drawing canvas - no questions asked
     * @private
     * @return {?}
     */
    clearCanvas() {
        this.drawingCtx.clearRect(0, 0, this.drawingCanvasRef.nativeElement.width, this.drawingCanvasRef.nativeElement.height);
    }
    /**
     * Draws commands on canvas
     * @private
     * @param {?} commands The draw commands to be drawn
     * @param {?} ctx The canvas 2d context
     * @return {?}
     */
    drawCommandsOnCanvas(commands, ctx) {
        commands.forEach((/**
         * @param {?} command
         * @return {?}
         */
        command => {
            command.draw(ctx);
        }));
    }
    /**
     * @param {?} command
     * @return {?}
     */
    addCommand(command) {
        this._drawCommands.push(command);
        this._redoCommands = [];
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    touchStart(ev) {
        /** @type {?} */
        const contactEvent = this.eventTranslator.translate(ev, this.canvasRect, this.projectionFactor);
        this._state.contactStart(this, contactEvent);
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    touchMove(ev) {
        ev.preventDefault(); // Disable scrolling
        // Disable scrolling
        /** @type {?} */
        const contactEvent = this.eventTranslator.translate(ev, this.canvasRect, this.projectionFactor);
        this._state.contactMove(this, contactEvent);
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    touchEnd(ev) {
        /** @type {?} */
        const contactEvent = this.eventTranslator.translate(ev, this.canvasRect, this.projectionFactor);
        this._state.contactEnd(this, contactEvent);
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    mouseDown(ev) {
        this.mouseCurrentlyDown = true;
        /** @type {?} */
        const contactEvent = this.eventTranslator.translate(ev, this.canvasRect, this.projectionFactor);
        this._state.contactStart(this, contactEvent);
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    mouseMove(ev) {
        if (!this.mouseCurrentlyDown) {
            return;
        }
        /** @type {?} */
        const contactEvent = this.eventTranslator.translate(ev, this.canvasRect, this.projectionFactor);
        this._state.contactMove(this, contactEvent);
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    mouseUp(ev) {
        this.mouseCurrentlyDown = false;
        /** @type {?} */
        const contactEvent = this.eventTranslator.translate(ev, this.canvasRect, this.projectionFactor);
        this._state.contactEnd(this, contactEvent);
    }
}
AiaImageAnnotatorComponent.decorators = [
    { type: Component, args: [{
                selector: 'aia-image-annotator',
                template: "<div class=\"outer-container\">\n    <div [style.width.px]=\"displayWidth\" [style.height.px]=\"displayHeight\" class=\"canvas-container\">\n        <canvas class=\"image-canvas\" #imageCanvas [width]=\"imageWidth\" [height]=\"imageHeight\" [style.width.px]=\"displayWidth\" [style.height.px]=\"displayHeight\"></canvas>\n        <canvas #drawingCanvas (touchstart)=\"touchStart($event)\" (touchmove)=\"touchMove($event)\" (touchend)=\"touchEnd($event)\"  (mousedown)=\"mouseDown($event)\" (mousemove)=\"mouseMove($event)\" (mouseup)=\"mouseUp($event)\" [width]=\"imageWidth\" [height]=\"imageHeight\" [style.width.px]=\"displayWidth\" [style.height.px]=\"displayHeight\"></canvas>\n        <canvas #mergeCanvas class=\"merge-canvas\" [width]=\"imageWidth\" [height]=\"imageHeight\"></canvas>\n        <aia-floating-text-entry (mouseup)=\"mouseUp($event)\" (touchend)=\"touchEnd($event)\"></aia-floating-text-entry>\n    </div>\n</div>\n",
                styles: [".outer-container{display:flex;justify-content:space-around}.outer-container .canvas-container{position:relative}.outer-container .canvas-container canvas{position:absolute;top:0;left:0}.outer-container .canvas-container canvas.image-canvas{z-index:-1}.outer-container .canvas-container canvas.merge-canvas{display:none}"]
            }] }
];
/** @nocollapse */
AiaImageAnnotatorComponent.ctorParameters = () => [];
AiaImageAnnotatorComponent.propDecorators = {
    image: [{ type: Input }],
    fontFamily: [{ type: Input }],
    fontSize: [{ type: Input }],
    color: [{ type: Input }],
    displayWidth: [{ type: Input }],
    imageCanvasRef: [{ type: ViewChild, args: ['imageCanvas',] }],
    drawingCanvasRef: [{ type: ViewChild, args: ['drawingCanvas',] }],
    mergeCanvasRef: [{ type: ViewChild, args: ['mergeCanvas',] }],
    textEntry: [{ type: ViewChild, args: [FloatingTextEntryComponent,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularImageAnnotatorModule {
}
AngularImageAnnotatorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AiaImageAnnotatorComponent, FloatingTextEntryComponent],
                imports: [
                    FormsModule
                ],
                exports: [AiaImageAnnotatorComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AiaImageAnnotatorComponent, AngularImageAnnotatorModule, FloatingTextEntryComponent as ɵa };

//# sourceMappingURL=angular-image-annotator.js.map